import { Component, OnInit, Input, forwardRef } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

@Component({
	selector: 'app-checkbutton',
	templateUrl: './checkbutton.component.html',
	styleUrls: ['./checkbutton.component.scss'],
	providers: [
		{
			provide: NG_VALUE_ACCESSOR,
			useExisting: forwardRef(() => CheckbuttonComponent),
			multi: true
		}
	]
})
export class CheckbuttonComponent implements OnInit, ControlValueAccessor {

	@Input()
	label?: string;

	@Input()
	value?: string | boolean = true;

	@Input()
	isDisabled?: boolean = false;
	
	@Input()
	meta = {};

	isChecked: boolean;
	currentValue: string | boolean;

	propagateChange = (_: any) => {};

	ngOnInit() {
		// meta input overrides other inputs if provided
		['label', 'value', 'isDisabled'].map(p => this[p] = this.meta[p] || this[p]);
	}
	
	private toggleChecked(e): void {
		e.target.blur();
		e.preventDefault();
		if (this.isDisabled) return;
		this.isChecked = !this.isChecked;
		this.currentValue = this.isChecked ? this.value : false;
		this.propagateChange(this.currentValue);
	}

	public writeValue(value: any): void {
		this.isChecked = value && value !== 0 && value !== 'false' ? value : false;
		this.currentValue = this.isChecked ? value : false;
	}

	public registerOnChange(fn: any): void {
		this.propagateChange = fn;
	}

	public registerOnTouched(fn: any): void {}

}
